@@ -26,9 +26,10 @@
 #include "ark_debug.h"
 #include "archive_kerfuffle.h"
 #include "createdialog.h"
+#include "settingsdialog.h"
 #include "settingspage.h"
 #include "pluginmanager.h"
-#include "part/interface.h"
+#include "interface.h"
 
 #include <KParts/ReadWritePart>
 #include <KPluginFactory>
@@ -73,7 +74,7 @@ MainWindow::~MainWindow()
 
     guiFactory()->removeClient(m_part);
     delete m_part;
-    m_part = 0;
+    m_part = nullptr;
 }
 
 void MainWindow::dragEnterEvent(QDragEnterEvent * event)
@@ -100,7 +101,7 @@ void MainWindow::dropEvent(QDropEvent * event)
         return;
     }
 
-    if ((event->source() == NULL) &&
+    if ((event->source() == nullptr) &&
         (isValidArchiveDrag(event->mimeData()))) {
         event->acceptProposedAction();
     }
@@ -119,7 +120,7 @@ void MainWindow::dragMoveEvent(QDragMoveEvent * event)
         return;
     }
 
-    if ((event->source() == NULL) &&
+    if ((event->source() == nullptr) &&
         (isValidArchiveDrag(event->mimeData()))) {
         event->acceptProposedAction();
     }
@@ -127,7 +128,7 @@ void MainWindow::dragMoveEvent(QDragMoveEvent * event)
 
 bool MainWindow::loadPart()
 {
-    KPluginFactory *factory = Q_NULLPTR;
+    KPluginFactory *factory = nullptr;
 
     const auto plugins = KPluginLoader::findPlugins(QString(), [](const KPluginMetaData& metaData) {
         return metaData.pluginId() == QStringLiteral("arkpart") &&
@@ -139,7 +140,7 @@ bool MainWindow::loadPart()
         factory = KPluginLoader(plugins.first().fileName()).factory();
     }
 
-    m_part = factory ? static_cast<KParts::ReadWritePart*>(factory->create<KParts::ReadWritePart>(this)) : Q_NULLPTR;
+    m_part = factory ? static_cast<KParts::ReadWritePart*>(factory->create<KParts::ReadWritePart>(this)) : nullptr;
 
     if (!m_part) {
         KMessageBox::error(this, i18n("Unable to find Ark's KPart component, please check your installation."));
@@ -170,7 +171,7 @@ void MainWindow::setupActions()
     m_openAction = actionCollection()->addAction(KStandardAction::Open, QStringLiteral("ark_file_open"), this, SLOT(openArchive()));
     actionCollection()->addAction(KStandardAction::Quit, QStringLiteral("ark_quit"), this, SLOT(quit()));
 
-    m_recentFilesAction = KStandardAction::openRecent(this, SLOT(openUrl(QUrl)), Q_NULLPTR);
+    m_recentFilesAction = KStandardAction::openRecent(this, SLOT(openUrl(QUrl)), nullptr);
     actionCollection()->addAction(QStringLiteral("ark_file_open_recent"), m_recentFilesAction);
 
     m_recentFilesAction->setToolBarMode(KRecentFilesAction::MenuMode);
@@ -257,10 +258,12 @@ void MainWindow::showSettings()
     Interface *iface = qobject_cast<Interface*>(m_part);
     Q_ASSERT(iface);
 
-    KConfigDialog *dialog = new KConfigDialog(this, QStringLiteral("settings"), iface->config());
+    auto dialog = new Kerfuffle::SettingsDialog(this, QStringLiteral("settings"), iface->config());
 
     foreach (Kerfuffle::SettingsPage *page, iface->settingsPages(this)) {
         dialog->addPage(page, page->name(), page->iconName());
+        connect(dialog, &KConfigDialog::settingsChanged, page, &Kerfuffle::SettingsPage::slotSettingsChanged);
+        connect(dialog, &Kerfuffle::SettingsDialog::defaultsButtonClicked, page, &Kerfuffle::SettingsPage::slotDefaultsButtonClicked);
     }
     // Hide the icons list if only one page has been added.
     dialog->setFaceType(KPageDialog::Auto);
@@ -285,7 +288,7 @@ void MainWindow::newArchive()
     Q_UNUSED(iface);
 
     QPointer<Kerfuffle::CreateDialog> dialog = new Kerfuffle::CreateDialog(
-        Q_NULLPTR, // parent
+        nullptr, // parent
         i18n("Create New Archive"), // caption
         QUrl()); // startDir
 
