@@ -28,18 +28,20 @@
 #include "overwritedialog.h"
 #include "archiveformat.h"
 #include "archivemodel.h"
+#include "archivesortfiltermodel.h"
 #include "archiveview.h"
 #include "arkviewer.h"
 #include "dnddbusinterfaceadaptor.h"
 #include "infopanel.h"
 #include "jobtracker.h"
 #include "generalsettingspage.h"
-#include "kerfuffle/extractiondialog.h"
-#include "kerfuffle/extractionsettingspage.h"
-#include "kerfuffle/jobs.h"
-#include "kerfuffle/settings.h"
-#include "kerfuffle/previewsettingspage.h"
-#include "kerfuffle/propertiesdialog.h"
+#include "extractiondialog.h"
+#include "extractionsettingspage.h"
+#include "jobs.h"
+#include "settings.h"
+#include "previewsettingspage.h"
+#include "propertiesdialog.h"
+#include "pluginsettingspage.h"
 #include "pluginmanager.h"
 
 #include <KAboutData>
@@ -75,6 +77,7 @@
 #include <QFileSystemWatcher>
 #include <QGroupBox>
 #include <QPlainTextEdit>
+#include <QPushButton>
 
 using namespace Kerfuffle;
 
@@ -87,9 +90,9 @@ static quint32 s_instanceCounter = 1;
 
 Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
         : KParts::ReadWritePart(parent),
-          m_splitter(Q_NULLPTR),
+          m_splitter(nullptr),
           m_busy(false),
-          m_jobTracker(Q_NULLPTR)
+          m_jobTracker(nullptr)
 {
     Q_UNUSED(args)
     KAboutData aboutData(QStringLiteral("ark"),
@@ -108,6 +111,7 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
     QWidget *mainWidget = new QWidget;
     m_vlayout = new QVBoxLayout;
     m_model = new ArchiveModel(pathName, this);
+    m_filterModel = new ArchiveSortFilterModel(this);
     m_splitter = new QSplitter(Qt::Horizontal, parentWidget);
     m_view = new ArchiveView;
     m_infoPanel = new InfoPanel(m_model);
@@ -143,6 +147,28 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
     setWidget(mainWidget);
     mainWidget->setLayout(m_vlayout);
 
+    // Setup search widget.
+    m_searchWidget = new QWidget(parentWidget);
+    m_searchWidget->setVisible(false);
+    m_searchWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
+    QHBoxLayout *searchLayout = new QHBoxLayout;
+    searchLayout->setContentsMargins(2, 2, 2, 2);
+    m_vlayout->addWidget(m_searchWidget);
+    m_searchWidget->setLayout(searchLayout);
+    m_searchCloseButton = new QPushButton(QIcon::fromTheme(QStringLiteral("dialog-close")), QString(), m_searchWidget);
+    m_searchCloseButton->setFlat(true);
+    m_searchLineEdit = new QLineEdit(m_searchWidget);
+    m_searchLineEdit->setClearButtonEnabled(true);
+    m_searchLineEdit->setPlaceholderText(i18n("Type to search..."));
+    mainWidget->installEventFilter(this);
+    searchLayout->addWidget(m_searchCloseButton);
+    searchLayout->addWidget(m_searchLineEdit);
+    connect(m_searchCloseButton, &QPushButton::clicked, this, [=]() {
+        m_searchWidget->hide();
+        m_searchLineEdit->clear();
+    });
+    connect(m_searchLineEdit, &QLineEdit::textChanged, this, &Part::searchEdited);
+
     // Configure the QVBoxLayout and add widgets
     m_vlayout->setContentsMargins(0,0,0,0);
     m_vlayout->addWidget(m_messageWidget);
@@ -228,7 +254,7 @@ void Part::registerJob(KJob* job)
 {
     if (!m_jobTracker) {
         m_jobTracker = new JobTracker(widget());
-        m_statusBarExtension->addStatusBarItem(m_jobTracker->widget(0), 0, true);
+        m_statusBarExtension->addStatusBarItem(m_jobTracker->widget(nullptr), 0, true);
         m_jobTracker->widget(job)->show();
     }
     m_jobTracker->registerJob(job);
@@ -274,7 +300,7 @@ void Part::extractSelectedFilesTo(const QString& localPath)
     options.setDragAndDropEnabled(true);
 
     // Create and start the ExtractJob.
-    ExtractJob *job = m_model->extractFiles(filesAndRootNodesForIndexes(addChildren(m_view->selectionModel()->selectedRows())), destination, options);
+    ExtractJob *job = m_model->extractFiles(filesAndRootNodesForIndexes(addChildren(getSelectedIndexes())), destination, options);
     registerJob(job);
     connect(job, &KJob::result,
             this, &Part::slotExtractionDone);
@@ -291,7 +317,10 @@ void Part::setupView()
 {
     m_view->setContextMenuPolicy(Qt::CustomContextMenu);
 
-    m_view->setModel(m_model);
+    m_filterModel->setSourceModel(m_model);
+    m_view->setModel(m_filterModel);
+    m_filterModel->setFilterKeyColumn(0);
+    m_filterModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
 
     connect(m_view->selectionModel(), &QItemSelectionModel::selectionChanged,
             this, &Part::updateActions);
@@ -428,6 +457,13 @@ void Part::setupActions()
     m_testArchiveAction->setToolTip(i18nc("@info:tooltip", "Click to test the archive for integrity"));
     connect(m_testArchiveAction, &QAction::triggered, this, &Part::slotTestArchive);
 
+    m_searchAction = actionCollection()->addAction(QStringLiteral("find_in_archive"));
+    m_searchAction->setIcon(QIcon::fromTheme(QStringLiteral("search")));
+    m_searchAction->setText(i18nc("@action:inmenu", "&Find Files"));
+    actionCollection()->setDefaultShortcut(m_searchAction, Qt::CTRL + Qt::Key_F);
+    m_searchAction->setToolTip(i18nc("@info:tooltip", "Click to search in archive"));
+    connect(m_searchAction, &QAction::triggered, this, &Part::slotShowFind);
+
     connect(m_signalMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
             this, &Part::slotOpenEntry);
 
@@ -439,7 +475,7 @@ void Part::setupActions()
 void Part::updateActions()
 {
     bool isWritable = m_model->archive() && !m_model->archive()->isReadOnly();
-    const Archive::Entry *entry = m_model->entryForIndex(m_view->selectionModel()->currentIndex());
+    const Archive::Entry *entry = m_model->entryForIndex(m_filterModel->mapToSource(m_view->selectionModel()->currentIndex()));
     int selectedEntriesCount = m_view->selectionModel()->selectedRows().count();
 
     // We disable adding files if the archive is encrypted but the password is
@@ -464,9 +500,9 @@ void Part::updateActions()
     // Figure out if entry size is larger than preview size limit.
     const int maxPreviewSize = ArkSettings::previewFileSizeLimit() * 1024 * 1024;
     const bool limit = ArkSettings::limitPreviewFileSize();
-    bool isPreviewable = (!limit || (limit && entry != Q_NULLPTR && entry->property("size").toLongLong() < maxPreviewSize));
+    bool isPreviewable = (!limit || (limit && entry != nullptr && entry->property("size").toLongLong() < maxPreviewSize));
 
-    const bool isDir = (entry == Q_NULLPTR) ? false : entry->isDir();
+    const bool isDir = (entry == nullptr) ? false : entry->isDir();
     m_previewAction->setEnabled(!isBusy() &&
                                 isPreviewable &&
                                 !isDir &&
@@ -508,6 +544,9 @@ void Part::updateActions()
                                    (selectedEntriesCount == 0 || (selectedEntriesCount == 1 && isDir)) &&
                                    (m_model->filesToMove.count() > 0 || m_model->filesToCopy.count() > 0));
 
+    m_searchAction->setEnabled(!isBusy() &&
+                               m_model->rowCount() > 0);
+
     m_commentView->setEnabled(!isBusy());
     m_commentMsgWidget->setEnabled(!isBusy());
 
@@ -537,7 +576,7 @@ void Part::slotShowComment()
 {
     if (!m_commentBox->isVisible()) {
         m_commentBox->show();
-        m_commentSplitter->setSizes(QList<int>() << m_view->height() * 0.6 << 1);
+        m_commentSplitter->setSizes(QList<int>() << static_cast<int>(m_view->height() * 0.6) << 1);
     }
     m_commentView->setFocus();
 }
@@ -657,7 +696,7 @@ void Part::updateQuickExtractMenu(QAction *extractAction)
     }
 
     while (menu->actions().size() > 3) {
-        menu->removeAction(menu->actions().last());
+        menu->removeAction(menu->actions().constLast());
     }
 
     const KConfigGroup conf(KSharedConfig::openConfig(), "ExtractDialog");
@@ -682,7 +721,7 @@ void Part::slotQuickExtractFiles(QAction *triggeredAction)
         const QString detectedSubfolder = detectSubfolder();
         qCDebug(ARK) << "Detected subfolder" << detectedSubfolder;
 
-        if (!isSingleFolderArchive()) {
+        if (m_model->archive()->hasMultipleTopLevelEntries()) {
             if (!userDestination.endsWith(QDir::separator())) {
                 userDestination.append(QDir::separator());
             }
@@ -694,7 +733,7 @@ void Part::slotQuickExtractFiles(QAction *triggeredAction)
 
         qCDebug(ARK) << "Extracting to:" << finalDestinationDirectory;
 
-        ExtractJob *job = m_model->extractFiles(filesAndRootNodesForIndexes(addChildren(m_view->selectionModel()->selectedRows())), finalDestinationDirectory, ExtractionOptions());
+        ExtractJob *job = m_model->extractFiles(filesAndRootNodesForIndexes(addChildren(getSelectedIndexes())), finalDestinationDirectory, ExtractionOptions());
         registerJob(job);
 
         connect(job, &KJob::result,
@@ -706,7 +745,16 @@ void Part::slotQuickExtractFiles(QAction *triggeredAction)
 
 void Part::selectionChanged()
 {
-    m_infoPanel->setIndexes(m_view->selectionModel()->selectedRows());
+    m_infoPanel->setIndexes(getSelectedIndexes());
+}
+
+QModelIndexList Part::getSelectedIndexes()
+{
+    QModelIndexList list;
+    foreach (const QModelIndex &i, m_view->selectionModel()->selectedRows()) {
+        list.append(m_filterModel->mapToSource(i));
+    }
+    return list;
 }
 
 bool Part::openFile()
@@ -750,6 +798,7 @@ QList<Kerfuffle::SettingsPage*> Part::settingsPages(QWidget *parent) const
     QList<SettingsPage*> pages;
     pages.append(new GeneralSettingsPage(parent, i18nc("@title:tab", "General Settings"), QStringLiteral("go-home")));
     pages.append(new ExtractionSettingsPage(parent, i18nc("@title:tab", "Extraction Settings"), QStringLiteral("archive-extract")));
+    pages.append(new PluginSettingsPage(parent, i18nc("@title:tab", "Plugin Settings"), QStringLiteral("plugins")));
     pages.append(new PreviewSettingsPage(parent, i18nc("@title:tab", "Preview Settings"), QStringLiteral("document-preview-archive")));
 
     return pages;
@@ -840,12 +889,7 @@ void Part::slotLoadingFinished(KJob *job)
     }
 
     m_view->sortByColumn(0, Qt::AscendingOrder);
-
-    // #303708: expand the first level only when there is just one root folder.
-    // Typical use case: an archive with source files.
-    if (m_view->model()->rowCount() == 1) {
-        m_view->expandToDepth(0);
-    }
+    m_view->expandIfSingleFolder();
 
     // After loading all files, resize the columns to fit all fields
     m_view->header()->resizeSections(QHeaderView::ResizeToContents);
@@ -922,7 +966,7 @@ void Part::slotOpenEntry(int mode)
 {
     qCDebug(ARK) << "Opening with mode" << mode;
 
-    QModelIndex index = m_view->selectionModel()->currentIndex();
+    QModelIndex index = m_filterModel->mapToSource(m_view->selectionModel()->currentIndex());
     Archive::Entry *entry = m_model->entryForIndex(index);
 
     // Don't open directories.
@@ -940,7 +984,7 @@ void Part::slotOpenEntry(int mode)
     if (!entry->fullPath().isEmpty()) {
 
         m_openFileMode = static_cast<OpenFileMode>(mode);
-        KJob *job = Q_NULLPTR;
+        KJob *job = nullptr;
 
         if (m_openFileMode == Preview) {
             job = m_model->preview(entry);
@@ -988,7 +1032,7 @@ void Part::slotOpenExtractedEntry(KJob *job)
         } else {
             KRun::runUrl(QUrl::fromUserInput(fullName, QString(), QUrl::AssumeLocalFile),
                          QMimeDatabase().mimeTypeForFile(fullName).name(),
-                         widget(), false, false);
+                         widget(), KRun::RunFlags());
         }
     } else if (job->error() != KJob::KilledJobError) {
         KMessageBox::error(widget(), job->errorString());
@@ -1043,7 +1087,7 @@ void Part::slotWatchedFileModified(const QString& file)
         QStringList list = QStringList() << file;
 
         qCDebug(ARK) << "Updating file" << file << "with path" << relPath;
-        slotAddFiles(list, Q_NULLPTR, relPath);
+        slotAddFiles(list, nullptr, relPath);
     }
     // This is needed because some apps, such as Kate, delete and recreate
     // files when saving.
@@ -1059,11 +1103,6 @@ void Part::slotError(const QString& errorMessage, const QString& details)
     }
 }
 
-bool Part::isSingleFolderArchive() const
-{
-    return m_model->archive()->isSingleFolder();
-}
-
 QString Part::detectSubfolder() const
 {
     if (!m_model) {
@@ -1096,7 +1135,7 @@ void Part::slotShowExtractionDialog()
         dialog.data()->setShowSelectedFiles(true);
     }
 
-    dialog.data()->setSingleFolderArchive(isSingleFolderArchive());
+    dialog.data()->setExtractToSubfolder(m_model->archive()->hasMultipleTopLevelEntries());
     dialog.data()->setSubfolder(detectSubfolder());
 
     dialog.data()->setCurrentUrl(QUrl::fromLocalFile(QFileInfo(m_model->archive()->fileName()).absolutePath()));
@@ -1114,7 +1153,7 @@ void Part::slotShowExtractionDialog()
         // If the user has chosen to extract only selected entries, fetch these
         // from the QTreeView.
         if (!dialog.data()->extractAllFiles()) {
-            files = filesAndRootNodesForIndexes(addChildren(m_view->selectionModel()->selectedRows()));
+            files = filesAndRootNodesForIndexes(addChildren(getSelectedIndexes()));
         }
 
         qCDebug(ARK) << "Selected " << files;
@@ -1218,7 +1257,7 @@ void Part::slotExtractionDone(KJob* job)
             QUrl destinationDirectory = QUrl::fromLocalFile(extractJob->destinationDirectory()).adjusted(QUrl::NormalizePathSegments);
             qCDebug(ARK) << "Shall open URL" << destinationDirectory;
 
-            KRun::runUrl(destinationDirectory, QStringLiteral("inode/directory"), widget());
+            KRun::runUrl(destinationDirectory, QStringLiteral("inode/directory"), widget(), KRun::RunExecutables, QString(), QByteArray());
         }
 
         if (ArkSettings::closeAfterExtraction()) {
@@ -1235,7 +1274,7 @@ void Part::slotAddFiles(const QStringList& filesToAdd, const Archive::Entry *des
 
     QStringList withChildPaths;
     foreach (const QString& file, filesToAdd) {
-        m_jobTempEntries.push_back(new Archive::Entry(Q_NULLPTR, file));
+        m_jobTempEntries.push_back(new Archive::Entry(nullptr, file));
         if (QFileInfo(file).isDir()) {
             withChildPaths << file + QLatin1Char('/');
             QDirIterator it(file, QDir::AllEntries | QDir::Readable | QDir::Hidden | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
@@ -1281,7 +1320,7 @@ void Part::slotAddFiles(const QStringList& filesToAdd, const Archive::Entry *des
         globalWorkDir.remove(relPath);
         qCDebug(ARK) << "Adding" << filesToAdd << "to" << relPath;
     } else {
-        qCDebug(ARK) << "Adding " << filesToAdd << ((destination == Q_NULLPTR) ? QString() : QStringLiteral("to ") + destination->fullPath());
+        qCDebug(ARK) << "Adding " << filesToAdd << ((destination == nullptr) ? QString() : QStringLiteral("to ") + destination->fullPath());
     }
 
     // Remove trailing slash (needed when adding dirs).
@@ -1324,7 +1363,7 @@ void Part::slotAddFiles()
         m_compressionOptions.setEncryptionMethod(arguments().metaData()[QStringLiteral("encryptionMethod")]);
     }
     if (!m_compressionOptions.isVolumeSizeSet() && arguments().metaData().contains(QStringLiteral("volumeSize"))) {
-        m_compressionOptions.setVolumeSize(arguments().metaData()[QStringLiteral("volumeSize")].toInt());
+        m_compressionOptions.setVolumeSize(arguments().metaData()[QStringLiteral("volumeSize")].toULong());
     }
 
     const auto compressionMethods = m_model->archive()->property("compressionMethods").toStringList();
@@ -1334,13 +1373,13 @@ void Part::slotAddFiles()
     }
 
     QString dialogTitle = i18nc("@title:window", "Add Files");
-    const Archive::Entry *destination = Q_NULLPTR;
+    const Archive::Entry *destination = nullptr;
     if (m_view->selectionModel()->selectedRows().count() == 1) {
-        destination = m_model->entryForIndex(m_view->selectionModel()->currentIndex());
+        destination = m_model->entryForIndex(m_filterModel->mapToSource(m_view->selectionModel()->currentIndex()));
         if (destination->isDir()) {
             dialogTitle = i18nc("@title:window", "Add Files to %1", destination->fullPath());;
         } else {
-            destination = Q_NULLPTR;
+            destination = nullptr;
         }
     }
 
@@ -1382,7 +1421,7 @@ void Part::slotEditFileName()
 
 void Part::slotCutFiles()
 {
-    QModelIndexList selectedRows = addChildren(m_view->selectionModel()->selectedRows());
+    QModelIndexList selectedRows = addChildren(getSelectedIndexes());
     m_model->filesToMove = ArchiveModel::entryMap(filesForIndexes(selectedRows));
     qCDebug(ARK) << "Entries marked to cut:" << m_model->filesToMove.values();
     m_model->filesToCopy.clear();
@@ -1398,7 +1437,7 @@ void Part::slotCutFiles()
 
 void Part::slotCopyFiles()
 {
-    m_model->filesToCopy = ArchiveModel::entryMap(filesForIndexes(addChildren(m_view->selectionModel()->selectedRows())));
+    m_model->filesToCopy = ArchiveModel::entryMap(filesForIndexes(addChildren(getSelectedIndexes())));
     qCDebug(ARK) << "Entries marked to copy:" << m_model->filesToCopy.values();
     foreach (const QModelIndex &row, m_cutIndexes) {
         m_view->dataChanged(row, row);
@@ -1414,8 +1453,8 @@ void Part::slotRenameFile(const QString &name)
         displayMsgWidget(KMessageWidget::Error, i18n("Filename can't contain slashes and can't be equal to \".\" or \"..\""));
         return;
     }
-    const Archive::Entry *entry = m_model->entryForIndex(m_view->selectionModel()->currentIndex());
-    QVector<Archive::Entry*> entriesToMove = filesForIndexes(addChildren(m_view->selectionModel()->selectedRows()));
+    const Archive::Entry *entry = m_model->entryForIndex(m_filterModel->mapToSource(m_view->selectionModel()->currentIndex()));
+    QVector<Archive::Entry*> entriesToMove = filesForIndexes(addChildren(getSelectedIndexes()));
 
     m_destination = new Archive::Entry();
     const QString &entryPath = entry->fullPath(NoTrailingSlash);
@@ -1432,12 +1471,12 @@ void Part::slotRenameFile(const QString &name)
 void Part::slotPasteFiles()
 {
     m_destination = (m_view->selectionModel()->selectedRows().count() > 0)
-                    ? m_model->entryForIndex(m_view->selectionModel()->currentIndex())
-                    : Q_NULLPTR;
-    if (m_destination == Q_NULLPTR) {
-        m_destination = new Archive::Entry(Q_NULLPTR, QString());
+                    ? m_model->entryForIndex(m_filterModel->mapToSource(m_view->selectionModel()->currentIndex()))
+                    : nullptr;
+    if (m_destination == nullptr) {
+        m_destination = new Archive::Entry(nullptr, QString());
     } else {
-        m_destination = new Archive::Entry(Q_NULLPTR, m_destination->fullPath());
+        m_destination = new Archive::Entry(nullptr, m_destination->fullPath());
     }
 
     if (m_model->filesToMove.count() > 0) {
@@ -1590,7 +1629,7 @@ void Part::slotDeleteFiles()
         return;
     }
 
-    DeleteJob *job = m_model->deleteFiles(filesForIndexes(addChildren(m_view->selectionModel()->selectedRows())));
+    DeleteJob *job = m_model->deleteFiles(filesForIndexes(addChildren(getSelectedIndexes())));
     connect(job, &KJob::result,
             this, &Part::slotDeleteFilesDone);
     registerJob(job);
@@ -1673,6 +1712,45 @@ void Part::slotShowContextMenu()
     popup->popup(QCursor::pos());
 }
 
+bool Part::eventFilter(QObject *target, QEvent *event)
+{
+    Q_UNUSED(target)
+
+    if (event->type() == QEvent::KeyPress) {
+        QKeyEvent *e = static_cast<QKeyEvent *>(event);
+        if (e->key() == Qt::Key_Escape) {
+            m_searchWidget->hide();
+            m_searchLineEdit->clear();
+            return true;
+        }
+    }
+    return false;
+}
+
+void Part::slotShowFind()
+{
+    if (m_searchWidget->isVisible()) {
+        m_searchLineEdit->selectAll();
+    } else {
+        m_searchWidget->show();
+    }
+    m_searchLineEdit->setFocus();
+}
+
+void Part::searchEdited(const QString &text)
+{
+    m_view->collapseAll();
+
+    m_filterModel->setFilterFixedString(text);
+
+    if(text.isEmpty()) {
+        m_view->collapseAll();
+        m_view->expandIfSingleFolder();
+    } else {
+        m_view->expandAll();
+    }
+}
+
 void Part::displayMsgWidget(KMessageWidget::MessageType type, const QString& msg)
 {
     // The widget could be already visible, so hide it.
